"use strict";
/*
    This file is part of MUTE-structs.

    Copyright (C) 2017  Matthieu Nicolas, Victorien Elvinger

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var data_validation_1 = require("../data-validation");
var identifier_1 = require("../identifier");
var identifierinterval_1 = require("../identifierinterval");
var idfactory_1 = require("../idfactory");
var int32_1 = require("../int32");
var RenamingMap = /** @class */ (function () {
    function RenamingMap(replicaNumber, clock, renamedIdIntervals) {
        var _this = this;
        this.replicaNumber = replicaNumber;
        this.clock = clock;
        this.renamedIdIntervals = renamedIdIntervals;
        this.indexes = [];
        var index = 0;
        renamedIdIntervals.forEach(function (idInterval) {
            _this.indexes.push(index);
            index += idInterval.length;
        });
        this.maxOffset = index - 1;
    }
    RenamingMap.fromPlain = function (o) {
        if (data_validation_1.isObject(o) &&
            int32_1.isInt32(o.replicaNumber) && int32_1.isInt32(o.clock) &&
            Array.isArray(o.renamedIdIntervals) &&
            o.renamedIdIntervals.length > 0) {
            var renamedIdIntervals = o.renamedIdIntervals
                .map(identifierinterval_1.IdentifierInterval.fromPlain)
                .filter(function (v) { return v !== null; });
            if (o.renamedIdIntervals.length === renamedIdIntervals.length) {
                return new RenamingMap(o.replicaNumber, o.clock, renamedIdIntervals);
            }
        }
        return null;
    };
    Object.defineProperty(RenamingMap.prototype, "firstId", {
        get: function () {
            return this.renamedIdIntervals[0].idBegin;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RenamingMap.prototype, "lastId", {
        get: function () {
            return this.renamedIdIntervals[this.renamedIdIntervals.length - 1].idEnd;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RenamingMap.prototype, "newFirstId", {
        get: function () {
            return idfactory_1.createAtPosition(this.replicaNumber, this.clock, this.newRandom, 0);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RenamingMap.prototype, "newLastId", {
        get: function () {
            return idfactory_1.createAtPosition(this.replicaNumber, this.clock, this.newRandom, this.maxOffset);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RenamingMap.prototype, "newRandom", {
        get: function () {
            return this.firstId.tuples[0].random;
        },
        enumerable: true,
        configurable: true
    });
    RenamingMap.prototype.renameId = function (id) {
        var minFirstId = this.firstId.compareTo(this.newFirstId) === -1 /* Less */ ?
            this.firstId : this.newFirstId;
        var maxLastId = this.lastId.compareTo(this.newLastId) === 1 /* Greater */ ?
            this.lastId : this.newLastId;
        if (id.compareTo(minFirstId) === -1 /* Less */ || maxLastId.compareTo(id) === -1 /* Less */) {
            return id;
        }
        if (id.compareTo(this.firstId) === -1 /* Less */) {
            // newFirstId < id < firstId
            // Happens if id.random = firstId.random && id.replicaNumber < firstId.replicaNumber
            var closestPredecessorOfNewFirstId = identifier_1.Identifier.fromBase(this.newFirstId, this.newFirstId.lastOffset - 1);
            return closestPredecessorOfNewFirstId.concat(id);
        }
        var _a = __read(this.findIndexOfIdOrPredecessor(id), 2), found = _a[0], index = _a[1];
        if (found) {
            return idfactory_1.createAtPosition(this.replicaNumber, this.clock, this.newRandom, index);
        }
        else {
            var newPredecessorId = idfactory_1.createAtPosition(this.replicaNumber, this.clock, this.newRandom, index);
            return newPredecessorId.concat(id);
        }
    };
    RenamingMap.prototype.renameIdInterval = function (idInterval) {
        var idBegin = idInterval.idBegin;
        var idEnd = idInterval.idEnd;
        var minFirstId = this.firstId.compareTo(this.newFirstId) === -1 /* Less */ ?
            this.firstId : this.newFirstId;
        var maxLastId = this.lastId.compareTo(this.newLastId) === 1 /* Greater */ ?
            this.lastId : this.newLastId;
        if (idEnd.compareTo(minFirstId) === -1 /* Less */ || maxLastId.compareTo(idBegin) === -1 /* Less */) {
            return [idInterval];
        }
        if (idEnd.compareTo(this.firstId) === -1 /* Less */) {
            // newFirstId < idInterval < firstId
            // Happens if id.random = firstId.random && id.replicaNumber < firstId.replicaNumber
            var closestPredecessorOfNewFirstId = identifier_1.Identifier.fromBase(this.newFirstId, this.newFirstId.lastOffset - 1);
            var newIdBegin = closestPredecessorOfNewFirstId.concat(idBegin);
            return [new identifierinterval_1.IdentifierInterval(newIdBegin, idInterval.end)];
        }
        var idIntervalIndex = this.findIndexOfRelevantIdInterval(idBegin);
        var relevantIdInterval = this.renamedIdIntervals[idIntervalIndex];
        if (relevantIdInterval.idEnd.compareTo(idBegin) === -1 /* Less */) {
            var newOffset = this.indexes[idIntervalIndex] + relevantIdInterval.length - 1;
            var newPredecessorId = idfactory_1.createAtPosition(this.replicaNumber, this.clock, this.newRandom, newOffset);
            var newIdBegin = newPredecessorId.concat(idBegin);
            return [new identifierinterval_1.IdentifierInterval(newIdBegin, idInterval.end)];
        }
        else {
            var offset = idBegin.tuples[relevantIdInterval.idBegin.length - 1].offset;
            var diff = offset - relevantIdInterval.begin;
            var newOffset = this.indexes[idIntervalIndex] + diff;
            if (idBegin.length === relevantIdInterval.idBegin.length) {
                var newIdBegin = idfactory_1.createAtPosition(this.replicaNumber, this.clock, this.newRandom, newOffset);
                if (relevantIdInterval.end < idInterval.end) {
                    var newEnd = this.indexes[idIntervalIndex + 1] - 1;
                    var newIdInterval = new identifierinterval_1.IdentifierInterval(newIdBegin, newEnd);
                    var toto = identifier_1.Identifier.fromBase(idBegin, relevantIdInterval.end + 1);
                    var titi = newIdInterval.idEnd.concat(toto);
                    var tata = new identifierinterval_1.IdentifierInterval(titi, idInterval.end);
                    return [newIdInterval, tata];
                }
                else {
                    var newEnd = newOffset + idInterval.length - 1;
                    var newIdInterval = new identifierinterval_1.IdentifierInterval(newIdBegin, newEnd);
                    return [newIdInterval];
                }
            }
            else {
                // Split
                var newPredecessorId = idfactory_1.createAtPosition(this.replicaNumber, this.clock, this.newRandom, newOffset);
                var newIdBegin = newPredecessorId.concat(idBegin);
                return [new identifierinterval_1.IdentifierInterval(newIdBegin, idInterval.end)];
            }
        }
    };
    RenamingMap.prototype.reverseRenameId = function (id) {
        if (this.hasBeenRenamed(id)) {
            // id ∈ renamedIds
            return this.findIdFromIndex(id.lastOffset);
        }
        var closestPredecessorOfNewFirstId = identifier_1.Identifier.fromBase(this.newFirstId, this.newFirstId.lastOffset - 1);
        var minFirstId = this.firstId.compareTo(closestPredecessorOfNewFirstId) === -1 /* Less */ ?
            this.firstId : closestPredecessorOfNewFirstId;
        var maxLastId = this.lastId.compareTo(this.newLastId) === 1 /* Greater */ ?
            this.lastId : this.newLastId;
        if (id.compareTo(minFirstId) === -1 /* Less */
            || maxLastId.compareTo(id) === -1 /* Less */) {
            return id;
        }
        if (id.compareTo(this.newFirstId) === -1 /* Less */) {
            // closestPredecessorOfNewFirstId < id < newFirstId
            console.assert(this.newFirstId.compareTo(this.firstId) === -1 /* Less */, "Reaching this case should imply that newFirstId < firstId");
            var closestPredecessorOfFirstId = identifier_1.Identifier.fromBase(this.firstId, this.firstId.lastOffset - 1);
            var _a = __read(id.truncate(1), 2), _ = _a[0], end = _a[1];
            if (end.tuples[0].random === this.newRandom) {
                // This case corresponds to the following scenario:
                // - end was inserted concurrently to the renaming operation with
                //      newFirstId < end < firstId
                // - so with
                //      newFirst.random = end.random = firstId.random
                //   and
                //      newFirst.author < end.author < firstId.author
                // - id was thus obtained by concatenating closestPredecessorOfNewFirstId + end
                // To revert the renaming, just need to return end
                return end;
            }
            if (closestPredecessorOfFirstId.compareTo(end) === -1 /* Less */) {
                return closestPredecessorOfFirstId.concat(end);
            }
            return id;
        }
        if (this.newLastId.compareTo(id) === -1 /* Less */ &&
            id.compareTo(this.lastId) === -1 /* Less */) {
            // newLastId < id < lastId < lastId + MIN_TUPLE + id
            return new identifier_1.Identifier(__spread(this.lastId.tuples, [
                idfactory_1.MIN_TUPLE
            ], id.tuples));
        }
        // newFirstId < id < newLastId
        var _b = __read(id.truncate(1), 2), head = _b[0], tail = _b[1];
        var _c = __read(this.findPredecessorAndSuccessorFromIndex(head.lastOffset), 2), predecessorId = _c[0], successorId = _c[1];
        if (tail.compareTo(predecessorId) === -1 /* Less */) {
            // tail < predecessorId < predecessorId + MIN_TUPLE + tail < successorId
            return new identifier_1.Identifier(__spread(predecessorId.tuples, [
                idfactory_1.MIN_TUPLE
            ], tail.tuples));
        }
        else if (successorId.compareTo(tail) === -1 /* Less */) {
            // predecessorId < closestPredecessorOfSuccessorId + MAX_TUPLE + tail < successorId < tail
            var closestPredecessorOfSuccessorId = identifier_1.Identifier.fromBase(successorId, successorId.lastOffset - 1);
            return new identifier_1.Identifier(__spread(closestPredecessorOfSuccessorId.tuples, [
                idfactory_1.MAX_TUPLE
            ], tail.tuples));
        }
        return tail;
    };
    RenamingMap.prototype.hasBeenRenamed = function (id) {
        return id.equalsBase(this.newFirstId)
            && 0 <= id.lastOffset && id.lastOffset <= this.maxOffset;
    };
    RenamingMap.prototype.findIndexOfIdOrPredecessor = function (id) {
        var l = 0;
        var r = this.renamedIdIntervals.length;
        while (l < r) {
            var m = Math.floor((l + r) / 2);
            var other = this.renamedIdIntervals[m];
            if (other.idEnd.compareTo(id) === -1 /* Less */) {
                l = m + 1;
            }
            else if (id.compareTo(other.idBegin) === -1 /* Less */) {
                r = m;
            }
            else {
                // other.idBegin <= id <= other.idEnd
                // But could also means that id splits other
                var offset = id.tuples[other.idBegin.length - 1].offset;
                var diff = offset - other.begin;
                return [other.idBegin.length === id.length, this.indexes[m] + diff];
            }
        }
        // Could not find id in the renamedIdIntervals
        // Return the predecessor's index in this case
        if (this.indexes.length <= l) {
            // lastId < id
            return [false, this.maxOffset];
        }
        return [false, this.indexes[l] - 1];
    };
    RenamingMap.prototype.findIndexOfRelevantIdInterval = function (id) {
        var l = 0;
        var r = this.renamedIdIntervals.length;
        while (l < r) {
            var m = Math.floor((l + r) / 2);
            var other = this.renamedIdIntervals[m];
            if (other.idEnd.compareTo(id) === -1 /* Less */) {
                l = m + 1;
            }
            else if (id.compareTo(other.idBegin) === -1 /* Less */) {
                r = m;
            }
            else {
                return m;
            }
        }
        return l - 1;
    };
    RenamingMap.prototype.findIdFromIndex = function (index) {
        var _a = __read(this.findPositionFromIndex(index), 2), idIntervalIndex = _a[0], offset = _a[1];
        var idBegin = this.renamedIdIntervals[idIntervalIndex].idBegin;
        return identifier_1.Identifier.fromBase(idBegin, offset);
    };
    RenamingMap.prototype.findPredecessorAndSuccessorFromIndex = function (index) {
        var _a = __read(this.findPositionFromIndex(index), 2), predecessorIndex = _a[0], predecessorOffset = _a[1];
        var predecessorIdInterval = this.renamedIdIntervals[predecessorIndex];
        var predecessorId = identifier_1.Identifier.fromBase(predecessorIdInterval.idBegin, predecessorOffset);
        var successorId = predecessorOffset !== predecessorIdInterval.end ?
            identifier_1.Identifier.fromBase(predecessorId, predecessorOffset + 1) :
            this.renamedIdIntervals[predecessorIndex + 1].idBegin;
        return [predecessorId, successorId];
    };
    RenamingMap.prototype.findPositionFromIndex = function (index) {
        var l = 0;
        var r = this.renamedIdIntervals.length;
        while (l <= r) {
            var m = Math.floor((l + r) / 2);
            var otherIndex = this.indexes[m];
            var otherIdInterval = this.renamedIdIntervals[m];
            if (otherIndex + otherIdInterval.length <= index) {
                l = m + 1;
            }
            else if (index < otherIndex) {
                r = m;
            }
            else {
                var offset = index - otherIndex + otherIdInterval.begin;
                return [m, offset];
            }
        }
        throw Error("Should have found the id in the renamedIdIntervals");
    };
    return RenamingMap;
}());
exports.RenamingMap = RenamingMap;
//# sourceMappingURL=renamingmap.js.map